@@ -11,8 +11,8 @@
 #
 # See http://www.latex-project.org/lppl.txt for details.
 #
-# Running pdffonts on Linux
+# Running otfinfo on Linux
 #
 #CODEbegin
-pdffonts 02-03-3.pdf
+otfinfo -h
 #CODEend
