@@ -1,6 +1,6 @@
 -- merged file : lualibs-extended-merged.lua
 -- parent file : lualibs-extended.lua
--- merge date  : Wed Nov  6 19:11:53 2013
+-- merge date  : Mon May  5 07:37:22 2014
 
 do -- begin closure to overcome local limits and interference
 
@@ -21,15 +21,28 @@ local unpack,concat=table.unpack,table.concat
 local P,V,C,S,R,Ct,Cs,Cp,Carg,Cc=lpeg.P,lpeg.V,lpeg.C,lpeg.S,lpeg.R,lpeg.Ct,lpeg.Cs,lpeg.Cp,lpeg.Carg,lpeg.Cc
 local patterns,lpegmatch=lpeg.patterns,lpeg.match
 local utfchar,utfbyte=utf.char,utf.byte
-local loadstripped=_LUAVERSION<5.2 and load or function(str)
-  return load(dump(load(str),true)) 
+local loadstripped=nil
+if _LUAVERSION<5.2 then
+  loadstripped=function(str,shortcuts)
+    return load(str)
+  end
+else
+  loadstripped=function(str,shortcuts)
+    if shortcuts then
+      return load(dump(load(str),true),nil,nil,shortcuts)
+    else
+      return load(dump(load(str),true))
+    end
+  end
 end
 if not number then number={} end 
 local stripper=patterns.stripzeros
 local function points(n)
+  n=tonumber(n)
   return (not n or n==0) and "0pt" or lpegmatch(stripper,format("%.5fpt",n/65536))
 end
 local function basepoints(n)
+  n=tonumber(n)
   return (not n or n==0) and "0bp" or lpegmatch(stripper,format("%.5fbp",n*(7200/7227)/65536))
 end
 number.points=points
@@ -92,11 +105,39 @@ local pattern=Carg(1)/function(t)
 function strings.tabtospace(str,tab)
   return lpegmatch(pattern,str,1,tab or 7)
 end
-function strings.striplong(str) 
-  str=gsub(str,"^%s*","")
-  str=gsub(str,"[\n\r]+ *","\n")
-  return str
+local newline=patterns.newline
+local endofstring=patterns.endofstring
+local whitespace=patterns.whitespace
+local spacer=patterns.spacer
+local space=spacer^0
+local nospace=space/""
+local endofline=nospace*newline
+local stripend=(whitespace^1*endofstring)/""
+local normalline=(nospace*((1-space*(newline+endofstring))^1)*nospace)
+local stripempty=endofline^1/""
+local normalempty=endofline^1
+local singleempty=endofline*(endofline^0/"")
+local doubleempty=endofline*endofline^-1*(endofline^0/"")
+local stripstart=stripempty^0
+local p_prune_normal=Cs (stripstart*(stripend+normalline+normalempty )^0 )
+local p_prune_collapse=Cs (stripstart*(stripend+normalline+doubleempty )^0 )
+local p_prune_noempty=Cs (stripstart*(stripend+normalline+singleempty )^0 )
+local p_retain_normal=Cs ((normalline+normalempty )^0 )
+local p_retain_collapse=Cs ((normalline+doubleempty )^0 )
+local p_retain_noempty=Cs ((normalline+singleempty )^0 )
+local striplinepatterns={
+  ["prune"]=p_prune_normal,
+  ["prune and collapse"]=p_prune_collapse,
+  ["prune and no empty"]=p_prune_noempty,
+  ["retain"]=p_retain_normal,
+  ["retain and collapse"]=p_retain_collapse,
+  ["retain and no empty"]=p_retain_noempty,
+}
+strings.striplinepatterns=striplinepatterns
+function strings.striplines(str,how)
+  return str and lpegmatch(how and striplinepatterns[how] or p_prune_collapse,str) or str
 end
+strings.striplong=strings.striplines
 function strings.nice(str)
   str=gsub(str,"[:%-+_]+"," ") 
   return str
@@ -172,31 +213,58 @@ function number.sparseexponent(f,n)
   end
   return tostring(n)
 end
-local preamble=[[
-local type = type
-local tostring = tostring
-local tonumber = tonumber
-local format = string.format
-local concat = table.concat
-local signed = number.signed
-local points = number.points
-local basepoints = number.basepoints
-local utfchar = utf.char
-local utfbyte = utf.byte
-local lpegmatch = lpeg.match
-local nspaces = string.nspaces
-local tracedchar = string.tracedchar
-local autosingle = string.autosingle
-local autodouble = string.autodouble
-local sequenced = table.sequenced
-local formattednumber = number.formatted
-local sparseexponent = number.sparseexponent
-]]
 local template=[[
 %s
 %s
 return function(%s) return %s end
 ]]
+local preamble,environment="",{}
+if _LUAVERSION<5.2 then
+  preamble=[[
+local lpeg=lpeg
+local type=type
+local tostring=tostring
+local tonumber=tonumber
+local format=string.format
+local concat=table.concat
+local signed=number.signed
+local points=number.points
+local basepoints= number.basepoints
+local utfchar=utf.char
+local utfbyte=utf.byte
+local lpegmatch=lpeg.match
+local nspaces=string.nspaces
+local tracedchar=string.tracedchar
+local autosingle=string.autosingle
+local autodouble=string.autodouble
+local sequenced=table.sequenced
+local formattednumber=number.formatted
+local sparseexponent=number.sparseexponent
+    ]]
+else
+  environment={
+    global=global or _G,
+    lpeg=lpeg,
+    type=type,
+    tostring=tostring,
+    tonumber=tonumber,
+    format=string.format,
+    concat=table.concat,
+    signed=number.signed,
+    points=number.points,
+    basepoints=number.basepoints,
+    utfchar=utf.char,
+    utfbyte=utf.byte,
+    lpegmatch=lpeg.match,
+    nspaces=string.nspaces,
+    tracedchar=string.tracedchar,
+    autosingle=string.autosingle,
+    autodouble=string.autodouble,
+    sequenced=table.sequenced,
+    formattednumber=number.formatted,
+    sparseexponent=number.sparseexponent,
+  }
+end
 local arguments={ "a1" } 
 setmetatable(arguments,{ __index=function(t,k)
     local v=t[k-1]..",a"..k
@@ -235,7 +303,7 @@ local format_i=function(f)
   if f and f~="" then
     return format("format('%%%si',a%s)",f,n)
   else
-    return format("format('%%i',a%s)",n)
+    return format("format('%%i',a%s)",n) 
   end
 end
 local format_d=format_i
@@ -247,6 +315,10 @@ local format_f=function(f)
   n=n+1
   return format("format('%%%sf',a%s)",f,n)
 end
+local format_F=function(f)
+  n=n+1
+  return format("((a%s == 0 and '0') or (a%s == 1 and '1') or format('%%%sf',a%s))",n,n,f,n)
+end
 local format_g=function(f)
   n=n+1
   return format("format('%%%sg',a%s)",f,n)
@@ -461,7 +533,7 @@ local builder=Cs { "start",
     (
       P("%")/""*(
         V("!") 
-+V("s")+V("q")+V("i")+V("d")+V("f")+V("g")+V("G")+V("e")+V("E")+V("x")+V("X")+V("o")
++V("s")+V("q")+V("i")+V("d")+V("f")+V("F")+V("g")+V("G")+V("e")+V("E")+V("x")+V("X")+V("o")
 +V("c")+V("C")+V("S") 
 +V("Q") 
 +V("N")
@@ -481,6 +553,7 @@ local builder=Cs { "start",
   ["i"]=(prefix_any*P("i"))/format_i,
   ["d"]=(prefix_any*P("d"))/format_d,
   ["f"]=(prefix_any*P("f"))/format_f,
+  ["F"]=(prefix_any*P("F"))/format_F,
   ["g"]=(prefix_any*P("g"))/format_g,
   ["G"]=(prefix_any*P("G"))/format_G,
   ["e"]=(prefix_any*P("e"))/format_e,
@@ -518,8 +591,8 @@ local builder=Cs { "start",
   ["!"]=Carg(2)*prefix_any*P("!")*C((1-P("!"))^1)*P("!")/format_extension,
 }
 local direct=Cs (
-    P("%")/""*Cc([[local format = string.format return function(str) return format("%]])*(S("+- .")+R("09"))^0*S("sqidfgGeExXo")*Cc([[",str) end]])*P(-1)
-  )
+  P("%")*(S("+- .")+R("09"))^0*S("sqidfgGeExXo")*P(-1)/[[local format = string.format return function(str) return format("%0",str) end]]
+)
 local function make(t,str)
   local f
   local p
@@ -528,10 +601,10 @@ local function make(t,str)
     f=loadstripped(p)()
   else
     n=0
-    p=lpegmatch(builder,str,1,"..",t._extensions_) 
+    p=lpegmatch(builder,str,1,t._connector_,t._extensions_) 
     if n>0 then
       p=format(template,preamble,t._preamble_,arguments[n],p)
-      f=loadstripped(p)()
+      f=loadstripped(p,t._environment_)() 
     else
       f=function() return str end
     end
@@ -543,10 +616,22 @@ local function use(t,fmt,...)
   return t[fmt](...)
 end
 strings.formatters={}
-function strings.formatters.new()
-  local t={ _extensions_={},_preamble_="",_type_="formatter" }
-  setmetatable(t,{ __index=make,__call=use })
-  return t
+if _LUAVERSION<5.2 then
+  function strings.formatters.new(noconcat)
+    local t={ _type_="formatter",_connector_=noconcat and "," or "..",_extensions_={},_preamble_=preamble,_environment_={} }
+    setmetatable(t,{ __index=make,__call=use })
+    return t
+  end
+else
+  function strings.formatters.new(noconcat)
+    local e={} 
+    for k,v in next,environment do
+      e[k]=v
+    end
+    local t={ _type_="formatter",_connector_=noconcat and "," or "..",_extensions_={},_preamble_="",_environment_=e }
+    setmetatable(t,{ __index=make,__call=use })
+    return t
+  end
 end
 local formatters=strings.formatters.new() 
 string.formatters=formatters 
@@ -554,8 +639,12 @@ string.formatter=function(str,...) return formatters[str](...) end
 local function add(t,name,template,preamble)
   if type(t)=="table" and t._type_=="formatter" then
     t._extensions_[name]=template or "%s"
-    if preamble then
+    if type(preamble)=="string" then
       t._preamble_=preamble.."\n"..t._preamble_ 
+    elseif type(preamble)=="table" then
+      for k,v in next,preamble do
+        t._environment_[k]=v
+      end
     end
   end
 end
@@ -564,9 +653,15 @@ patterns.xmlescape=Cs((P("<")/"&lt;"+P(">")/"&gt;"+P("&")/"&amp;"+P('"')/"&quot;
 patterns.texescape=Cs((C(S("#$%\\{}"))/"\\%1"+P(1))^0)
 patterns.luaescape=Cs(((1-S('"\n'))^1+P('"')/'\\"'+P('\n')/'\\n"')^0) 
 patterns.luaquoted=Cs(Cc('"')*((1-S('"\n'))^1+P('"')/'\\"'+P('\n')/'\\n"')^0*Cc('"'))
-add(formatters,"xml",[[lpegmatch(xmlescape,%s)]],[[local xmlescape = lpeg.patterns.xmlescape]])
-add(formatters,"tex",[[lpegmatch(texescape,%s)]],[[local texescape = lpeg.patterns.texescape]])
-add(formatters,"lua",[[lpegmatch(luaescape,%s)]],[[local luaescape = lpeg.patterns.luaescape]])
+if _LUAVERSION<5.2 then
+  add(formatters,"xml",[[lpegmatch(xmlescape,%s)]],"local xmlescape = lpeg.patterns.xmlescape")
+  add(formatters,"tex",[[lpegmatch(texescape,%s)]],"local texescape = lpeg.patterns.texescape")
+  add(formatters,"lua",[[lpegmatch(luaescape,%s)]],"local luaescape = lpeg.patterns.luaescape")
+else
+  add(formatters,"xml",[[lpegmatch(xmlescape,%s)]],{ xmlescape=lpeg.patterns.xmlescape })
+  add(formatters,"tex",[[lpegmatch(texescape,%s)]],{ texescape=lpeg.patterns.texescape })
+  add(formatters,"lua",[[lpegmatch(luaescape,%s)]],{ luaescape=lpeg.patterns.luaescape })
+end
 
 end -- closure
 
@@ -591,27 +686,29 @@ local sortedkeys,sortedpairs=table.sortedkeys,table.sortedpairs
 local formatters=string.formatters
 local utftoeight=utf.toeight
 local splitter=lpeg.tsplitat(".")
-function tables.definetable(target,nofirst,nolast) 
-  local composed,shortcut,t=nil,nil,{}
+function utilities.tables.definetable(target,nofirst,nolast) 
+  local composed,t=nil,{}
   local snippets=lpegmatch(splitter,target)
   for i=1,#snippets-(nolast and 1 or 0) do
     local name=snippets[i]
     if composed then
-      composed=shortcut.."."..name
-      shortcut=shortcut.."_"..name
-      t[#t+1]=formatters["local %s = %s if not %s then %s = { } %s = %s end"](shortcut,composed,shortcut,shortcut,composed,shortcut)
+      composed=composed.."."..name
+        t[#t+1]=formatters["if not %s then %s = { } end"](composed,composed)
     else
       composed=name
-      shortcut=name
       if not nofirst then
         t[#t+1]=formatters["%s = %s or { }"](composed,composed)
       end
     end
   end
-  if nolast then
-    composed=shortcut.."."..snippets[#snippets]
+  if composed then
+    if nolast then
+      composed=composed.."."..snippets[#snippets]
+    end
+    return concat(t,"\n"),composed 
+  else
+    return "",target
   end
-  return concat(t,"\n"),composed
 end
 function tables.definedtable(...)
   local t=_G
@@ -638,7 +735,7 @@ function tables.accesstable(target,root)
 end
 function tables.migratetable(target,v,root)
   local t=root or _G
-  local names=string.split(target,".")
+  local names=lpegmatch(splitter,target)
   for i=1,#names-1 do
     local name=names[i]
     t[name]=t[name] or {}
@@ -805,7 +902,7 @@ local f_ordered_string=formatters["%q,"]
 local f_ordered_number=formatters["%s,"]
 local f_ordered_boolean=formatters["%l,"]
 function table.fastserialize(t,prefix)
-  local r={ prefix or "return" }
+  local r={ type(prefix)=="string" and prefix or "return" }
   local m=1
   local function fastserialize(t,outer) 
     local n=#t
@@ -1462,13 +1559,15 @@ function parsers.settings_to_array(str,strict)
   elseif not str or str=="" then
     return {}
   elseif strict then
-    if find(str,"{") then
+    if find(str,"{",1,true) then
       return lpegmatch(pattern,str)
     else
       return { str }
     end
-  else
+  elseif find(str,",",1,true) then
     return lpegmatch(pattern,str)
+  else
+    return { str }
   end
 end
 local separator=space^0*comma*space^0
@@ -1678,8 +1777,8 @@ function parsers.rfc4180splitter(specification)
   local separator=S(separator~="" and separator or ",")
   local escaped=quotechar*Cs((dquotechar+(1-quotechar))^0)*quotechar
   local non_escaped=C((1-quotechar-newline-separator)^1)
-  local field=escaped+non_escaped
-  local record=Ct((field*separator^-1)^1)
+  local field=escaped+non_escaped+Cc("")
+  local record=Ct(field*(separator*field)^1)
   local headerline=record*Cp()
   local wholeblob=Ct((newline^-1*record)^0)
   return function(data,getheader)
@@ -2101,7 +2200,7 @@ if not modules then modules={} end modules ['trac-inf']={
   license="see context related readme files"
 }
 local type,tonumber,select=type,tonumber,select
-local format,lower=string.format,string.lower
+local format,lower,find=string.format,string.lower,string.find
 local concat=table.concat
 local clock=os.gettimeofday or os.clock 
 local setmetatableindex=table.setmetatableindex
@@ -2191,6 +2290,10 @@ local report=logs.reporter("mkiv lua stats")
 function statistics.show()
   if statistics.enable then
     local register=statistics.register
+    register("used platform",function()
+      return format("%s, type: %s, binary subtree: %s",
+        os.platform or "unknown",os.type or "unknown",environment.texos or "unknown")
+    end)
     register("luatex banner",function()
       return lower(status.banner)
     end)
@@ -2202,14 +2305,23 @@ function statistics.show()
       return format("%s direct, %s indirect, %s total",total-indirect,indirect,total)
     end)
     if jit then
-      local status={ jit.status() }
-      if status[1] then
-        register("luajit status",function()
-          return concat(status," ",2)
-        end)
+      local jitstatus={ jit.status() }
+      if jitstatus[1] then
+        register("luajit options",concat(jitstatus," ",2))
       end
     end
-    register("current memory usage",statistics.memused)
+    register("lua properties",function()
+      local list=status.list()
+      local hashchar=tonumber(list.luatex_hashchars)
+      local mask=lua.mask or "ascii"
+      return format("engine: %s, used memory: %s, hash type: %s, hash chars: min(%s,40), symbol mask: %s (%s)",
+        jit and "luajit" or "lua",
+        statistics.memused(),
+        list.luatex_hashtype or "default",
+        hashchar and 2^hashchar or "unknown",
+        mask,
+        mask=="utf" and "τεχ" or "tex")
+    end)
     register("runtime",statistics.runtime)
     logs.newline() 
     for i=1,#statusinfo do
@@ -2239,7 +2351,7 @@ function statistics.timed(action)
   starttiming("run")
   action()
   stoptiming("run")
-  report("total runtime: %s",elapsedtime("run"))
+  report("total runtime: %s seconds",elapsedtime("run"))
 end
 function statistics.tracefunction(base,tag,...)
   for i=1,select("#",...) do
@@ -2468,20 +2580,22 @@ end
 function debugger.disable()
   debug.sethook()
 end
-function traceback()
-  local level=1
+local function showtraceback(rep) 
+  local level=2 
+  local reporter=rep or report
   while true do
-    local info=debug.getinfo(level,"Sl")
+    local info=getinfo(level,"Sl")
     if not info then
       break
     elseif info.what=="C" then
-      print(format("%3i : C function",level))
+      reporter("%2i : %s",level-1,"C function")
     else
-      print(format("%3i : [%s]:%d",level,info.short_src,info.currentline))
+      reporter("%2i : %s : %s",level-1,info.short_src,info.currentline)
     end
     level=level+1
   end
 end
+debugger.showtraceback=showtraceback
 
 end -- closure
 
@@ -2834,195 +2948,3 @@ function stacker.new(name)
 end
 
 end -- closure
-
-do -- begin closure to overcome local limits and interference
-
-if not modules then modules={} end modules ['util-env']={
-  version=1.001,
-  comment="companion to luat-lib.mkiv",
-  author="Hans Hagen, PRAGMA-ADE, Hasselt NL",
-  copyright="PRAGMA ADE / ConTeXt Development Team",
-  license="see context related readme files"
-}
-local allocate,mark=utilities.storage.allocate,utilities.storage.mark
-local format,sub,match,gsub,find=string.format,string.sub,string.match,string.gsub,string.find
-local unquoted,quoted=string.unquoted,string.quoted
-local concat,insert,remove=table.concat,table.insert,table.remove
-environment=environment or {}
-local environment=environment
-os.setlocale(nil,nil) 
-function os.setlocale()
-end
-local validengines=allocate {
-  ["luatex"]=true,
-  ["luajittex"]=true,
-}
-local basicengines=allocate {
-  ["luatex"]="luatex",
-  ["texlua"]="luatex",
-  ["texluac"]="luatex",
-  ["luajittex"]="luajittex",
-  ["texluajit"]="luajittex",
-}
-local luaengines=allocate {
-  ["lua"]=true,
-  ["luajit"]=true,
-}
-environment.validengines=validengines
-environment.basicengines=basicengines
-if not arg then
-  environment.used_as_library=true
-elseif luaengines[file.removesuffix(arg[-1])] then
-elseif validengines[file.removesuffix(arg[0])] then
-  if arg[1]=="--luaonly" then
-    arg[-1]=arg[0]
-    arg[ 0]=arg[2]
-    for k=3,#arg do
-      arg[k-2]=arg[k]
-    end
-    remove(arg) 
-    remove(arg) 
-  else
-  end
-  local originalzero=file.basename(arg[0])
-  local specialmapping={ luatools=="base" }
-  if originalzero~="mtxrun" and originalzero~="mtxrun.lua" then
-    arg[0]=specialmapping[originalzero] or originalzero
-    insert(arg,0,"--script")
-    insert(arg,0,"mtxrun")
-  end
-end
-environment.arguments=allocate()
-environment.files=allocate()
-environment.sortedflags=nil
-function environment.initializearguments(arg)
-  local arguments,files={},{}
-  environment.arguments,environment.files,environment.sortedflags=arguments,files,nil
-  for index=1,#arg do
-    local argument=arg[index]
-    if index>0 then
-      local flag,value=match(argument,"^%-+(.-)=(.-)$")
-      if flag then
-        flag=gsub(flag,"^c:","")
-        arguments[flag]=unquoted(value or "")
-      else
-        flag=match(argument,"^%-+(.+)")
-        if flag then
-          flag=gsub(flag,"^c:","")
-          arguments[flag]=true
-        else
-          files[#files+1]=argument
-        end
-      end
-    end
-  end
-  environment.ownname=file.reslash(environment.ownname or arg[0] or 'unknown.lua')
-end
-function environment.setargument(name,value)
-  environment.arguments[name]=value
-end
-function environment.getargument(name,partial)
-  local arguments,sortedflags=environment.arguments,environment.sortedflags
-  if arguments[name] then
-    return arguments[name]
-  elseif partial then
-    if not sortedflags then
-      sortedflags=allocate(table.sortedkeys(arguments))
-      for k=1,#sortedflags do
-        sortedflags[k]="^"..sortedflags[k]
-      end
-      environment.sortedflags=sortedflags
-    end
-    for k=1,#sortedflags do
-      local v=sortedflags[k]
-      if find(name,v) then
-        return arguments[sub(v,2,#v)]
-      end
-    end
-  end
-  return nil
-end
-environment.argument=environment.getargument
-function environment.splitarguments(separator) 
-  local done,before,after=false,{},{}
-  local originalarguments=environment.originalarguments
-  for k=1,#originalarguments do
-    local v=originalarguments[k]
-    if not done and v==separator then
-      done=true
-    elseif done then
-      after[#after+1]=v
-    else
-      before[#before+1]=v
-    end
-  end
-  return before,after
-end
-function environment.reconstructcommandline(arg,noquote)
-  arg=arg or environment.originalarguments
-  if noquote and #arg==1 then
-    local a=arg[1]
-    a=resolvers.resolve(a)
-    a=unquoted(a)
-    return a
-  elseif #arg>0 then
-    local result={}
-    for i=1,#arg do
-      local a=arg[i]
-      a=resolvers.resolve(a)
-      a=unquoted(a)
-      a=gsub(a,'"','\\"') 
-      if find(a," ") then
-        result[#result+1]=quoted(a)
-      else
-        result[#result+1]=a
-      end
-    end
-    return concat(result," ")
-  else
-    return ""
-  end
-end
-function environment.relativepath(path,root)
-  if not path then
-    path=""
-  end
-  if not file.is_rootbased_path(path) then
-    if not root then
-      root=file.pathpart(environment.ownscript or environment.ownname or ".")
-    end
-    if root=="" then
-      root="."
-    end
-    path=root.."/"..path
-  end
-  return file.collapsepath(path,true)
-end
-if arg then
-  local newarg,instring={},false
-  for index=1,#arg do
-    local argument=arg[index]
-    if find(argument,"^\"") then
-      newarg[#newarg+1]=gsub(argument,"^\"","")
-      if not find(argument,"\"$") then
-        instring=true
-      end
-    elseif find(argument,"\"$") then
-      newarg[#newarg]=newarg[#newarg].." "..gsub(argument,"\"$","")
-      instring=false
-    elseif instring then
-      newarg[#newarg]=newarg[#newarg].." "..argument
-    else
-      newarg[#newarg+1]=argument
-    end
-  end
-  for i=1,-5,-1 do
-    newarg[i]=arg[i]
-  end
-  environment.initializearguments(newarg)
-  environment.originalarguments=mark(newarg)
-  environment.rawarguments=mark(arg)
-  arg={} 
-end
-
-end -- closure
